﻿#region Using directives
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
#endregion

namespace Pic.DAL.SqlServer
{
   public interface TreeInterface
    {
        void Initialize();
        void Finish();
        object InsertTreeNode(object parentNode, TreeNode node);
        void AskRootNode();
    }

    public class TreeConsole : TreeInterface
    {
        public void Initialize()
        {
            Console.WriteLine("Initialize tree...");
        }

        public void Finish()
        {
            Console.WriteLine("Finalize tree...");
        }

        public object InsertTreeNode(object parentNode, TreeNode node)
        {
            string slineFrom = parentNode as string;
            Console.WriteLine(slineFrom + string.Format("Node : {0}({1})", node.Name, node.Description));
            string slineTo = slineFrom + "    ";
            return (object)(slineTo);
        }

        public void AskRootNode()
        { 
        }
    }

    public class TreeBuilder
    {
        public void BuildTree(TreeInterface treeImplementation)
        {
            // initialize
            treeImplementation.Initialize();
            // root nodes
            PPDataContext db = new PPDataContext();
            List<TreeNode> rootNodes = TreeNode.GetRootNodes(db);
            if (0 == rootNodes.Count)
            {
                treeImplementation.AskRootNode();
                rootNodes = TreeNode.GetRootNodes(db);
            }
            foreach (TreeNode node in rootNodes)
                InsertChildNodes(db, treeImplementation, null, node);     
            // finalize
            treeImplementation.Finish();
        }

        private void InsertChildNodes(PPDataContext db, TreeInterface treeImplementation, object parentNode, TreeNode node)
        {
            // node itself
            Object object1 = treeImplementation.InsertTreeNode(parentNode, node);
            // child nodes
            List<TreeNode> childNodes = node.Childrens(db);
            foreach (TreeNode tn in childNodes)
                InsertChildNodes(db, treeImplementation, object1, tn);
        }
    }
}
