﻿#region Using directives
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Drawing;
using System.IO;
using System.Web;
using System.Web.Caching;
using System.Web.SessionState;
using Pic.Plugin;

#endregion

namespace Pic.Param.HttpHandler
{
    class ImageHttpHandler : IHttpHandler
    {
		private double dTimeout = 120;
		private int width   = 512;
		private int height  = 512;

        public void ProcessRequest(HttpContext context)
        {
            //
            // invoke webservice to obtain images
            //
            Pic.Param.WebService.ParamService picParamServ = new Pic.Param.WebService.ParamService();

            string sName = context.Request.QueryString["name"];
            ParameterStack stack = picParamServ.GetParameters( sName );
            // retrieve parameter values from url
            foreach (Parameter param in stack.ParameterList)
            {
                if (param.GetType().ToString().Equals("Pic.Plugin.ParameterDouble"))
                {
                    ParameterDouble paramDouble = (ParameterDouble)param;
                    paramDouble.Value = double.Parse(context.Request.QueryString[param.Name]);
                }
                else if (param.GetType() == typeof(ParameterBool))
                {
                    ParameterBool paramBool = param as ParameterBool;
                    paramBool.Value = bool.Parse(context.Request.QueryString[param.Name]);
                }
            }

            // get image
            System.Byte[] imgModel = picParamServ.GetModelImage(width, height, sName, stack);
            MemoryStream memStream = new MemoryStream(imgModel);
            Bitmap Image = new Bitmap(memStream);

            // Store guid/keyword combination in the Cache and
            // set how many seconds it will be remain valid (dTimeout)
            HttpContext.Current.Cache.Insert("name", sName, null, DateTime.Now.AddSeconds(dTimeout), TimeSpan.Zero, CacheItemPriority.High, null);

            // write the image to the Http output stream
            Image.Save(context.Response.OutputStream, System.Drawing.Imaging.ImageFormat.Jpeg);

            context.Response.ContentType = "image/jpeg";
            context.Response.StatusCode = 200;

            context.Response.End();
        }

        public bool IsReusable
        {
            get
            {
                return true;
            }
        }
    }
}
