﻿#region Using directives
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Web;
using System.Web.Caching;
using Pic.Plugin;
using Pic.DAL.SqlServer;
#endregion

namespace Pic.Param.HttpHandlers
{
    #region ComponentSearchMethodDB method
    public class ComponentSearchMethodDB : IComponentSearchMethod
    {
        #region Constructors
        public ComponentSearchMethodDB()
        {
        }
        #endregion

        #region IComponentSearchMethod implementation
        public string GetFilePathFromGuid(Guid guid)
        {
            PPDataContext db = new PPDataContext();
            Pic.DAL.SqlServer.Component comp = Pic.DAL.SqlServer.Component.GetByGuid(db, guid);
            return comp.Document.File.Path;
        }
        public double GetDoubleParameterDefaultValue(Guid guid, string name)
        {
            return 0.0;
        }
        public int GetIntParameterDefaultValue(Guid guid, string name)
        {
            return 0;
        }
        public bool GetBoolParameterDefaultValue(Guid guid, string name)
        {
            return false;
        }
        public int GetMultiParameterDefaultValue(Guid guid, string name)
        {
            return 0;
        }
        #endregion
    }
    #endregion

    public class HttpHandlerImage : IHttpHandler
    {
        #region Data members
        private double dTimeout = 120;
        private int width = 512;
        private int height = 512;
        #endregion

        #region IHttpHandler implementation
        /// <summary>
        /// Process http request
        /// </summary>
        /// <param name="context"></param>
        public void ProcessRequest(HttpContext context)
        {
            try
            {
                // retrieve document
                string sDocumentId = context.Request.QueryString["DocumentId"];

                PPDataContext db = new PPDataContext();
                Document doc = Document.GetById(db, Convert.ToInt32(sDocumentId));
                // load plugin
                ComponentSearchMethodDB componentSearchMethod = new ComponentSearchMethodDB();
                ComponentLoader loader = new ComponentLoader();
                loader.SearchMethod = componentSearchMethod;

                Pic.Plugin.Component component = loader.LoadComponent(doc.File.Path);
                // retrieve parameter stack
                ParameterStack stack = component.BuildParameterStack(null);
                // retrieve parameter values from url
                foreach (Parameter param in stack.ParameterList)
                {
                    if (param.GetType().ToString().Equals("Pic.Plugin.ParameterDouble"))
                    {
                        ParameterDouble paramDouble = (ParameterDouble)param;
                        paramDouble.Value = double.Parse(context.Request.QueryString[param.Name]);
                    }
                    else if (param.GetType() == typeof(ParameterBool))
                    {
                        ParameterBool paramBool = param as ParameterBool;
                        paramBool.Value = bool.Parse(context.Request.QueryString[param.Name]);
                    }
                }
                // initialize 
                Tools pluginTools = new Tools(component, componentSearchMethod);
                pluginTools.ReflexionX = bool.Parse(context.Request.QueryString["ReflexionX"]);
                pluginTools.ReflexionY = bool.Parse(context.Request.QueryString["ReflexionY"]);
                pluginTools.ShowCotations = bool.Parse(context.Request.QueryString["ShowQuotations"]);
                // generate image
                Image bmp;
                pluginTools.GenerateImage(new Size(width, height), stack, out bmp);

                // Store guid/keyword combination in the Cache and
                // set how many seconds it will be remain valid (dTimeout)
                HttpContext.Current.Cache.Insert("DocumentId", sDocumentId, null, DateTime.Now.AddSeconds(dTimeout), TimeSpan.Zero, CacheItemPriority.High, null);
                // write the image to the output stream
                bmp.Save(context.Response.OutputStream, System.Drawing.Imaging.ImageFormat.Jpeg);
                context.Response.ContentType = "image/jpeg";
                context.Response.StatusCode = 200;
                context.Response.End();
            }
            catch (Exception ex)
            {
                context.Response.Write(ex.ToString());
            }
        }

        public bool IsReusable { get { return true; } }
        #endregion
    }
}
