﻿#region Using directives
using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Collections.Specialized;

using System.Web;
using System.Web.Security;

using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using Pic.Param.WebService;
using Pic.Plugin;
#endregion

namespace Pic.Param.WebApplication
{
    public partial class _Default : System.Web.UI.Page
    {
        string sModelName;
        ParameterStack paramStack;

        #region System.Web.UI.Page overrides
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            // get reference to web service 
            Pic.Param.WebService.ParamService picParamServ = new Pic.Param.WebService.ParamService();
            // get all model names from web service
            StringCollection listModel = picParamServ.GetListOfAvailableModels();
            // fill list box
            lstModels.Items.Clear();
            foreach (string modelName in listModel)
                lstModels.Items.Add(modelName);
            // get list of file format
            StringCollection listFormats = picParamServ.GetListOfOutputFileFormats();
            // fill combo box
            cbDownload.Items.Clear();
            foreach (string fileFormat in listFormats)
                cbDownload.Items.Add(fileFormat);
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);
            this.ViewState["ModelIndex"] = this.lstModels.SelectedIndex;
        }

        protected override void LoadViewState(object savedState)
        {
            base.LoadViewState(savedState);
            int index = (int)this.ViewState["ModelIndex"];

            if (this.lstModels.Items.Count > 0)
                this.lstModels.SelectedIndex = index;
        }
        #endregion

        #region Page load
        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                // clear error message
                LabelError.Text = "";

                // get the webservice
                Pic.Param.WebService.ParamService picParamServ = new Pic.Param.WebService.ParamService();
                // set the first model item as the default
                if (!this.IsPostBack && lstModels.Items.Count > 0)
                {
                    lstModels.SelectedIndex = 0;
                    cbDownload.SelectedIndex = 0;
                }

                sModelName = lstModels.Items[lstModels.SelectedIndex].Text;
                paramStack = picParamServ.GetParameters(sModelName);

                tblDynamic.Rows.Clear();

                foreach (Pic.Plugin.Parameter param in paramStack.ParameterList)
                {
                    TableRow tr = new TableRow();

                    if (param.GetType() == typeof(ParameterDouble))
                    {
                        // create column 1
                        TableCell td1 = new TableCell();
                        // create label control dynamically
                        Label label1 = new Label();
                        label1.ID = "lbl_ " + param.Name;
                        label1.Text = param.Name;
                        // add control to the table cell
                        td1.Controls.Add(label1);

                        // create column2
                        TableCell td2 = new TableCell();
                        // create control dynamically
                        TextBox tb1 = new TextBox();
                        tb1.Width = 50;
                        tb1.ID = "txt_" + param.Name;
                        ParameterDouble paramDouble = (ParameterDouble)param;
                        tb1.Text = paramDouble.ValueDefault.ToString();
                        tb1.ToolTip = paramDouble.Description;
                        // add control to the table cell
                        td2.Controls.Add(tb1);
                        // add cells to the row
                        tr.Cells.Add(td1);
                        tr.Cells.Add(td2);
                    }
                    else if (param.GetType() == typeof(ParameterBool))
                    {
                        ParameterBool paramBool = param as ParameterBool;
                        // create column1
                        TableCell td1 = new TableCell();
                        CheckBox cb1 = new CheckBox();
                        cb1.ID = "cb_" + param.Name;
                        cb1.Text = param.Name;
                        cb1.ToolTip = param.Description;
                        cb1.AutoPostBack = true;
                        cb1.Checked = paramBool.Value;
                        cb1.CheckedChanged += new EventHandler(btUpdate_Click);
                        // add control to the table cell
                        td1.Controls.Add(cb1);
                        // add cells to the row
                        tr.Cells.Add(td1);
                    }
                    else
                        throw new Exception("Unknown parameter type");

                    // add row to the table
                    tblDynamic.Rows.Add(tr);
                }

                // insert update button
                TableRow trow = new TableRow();
                TableCell tcell1 = new TableCell();
                trow.Cells.Add(tcell1);
                TableCell tcell2 = new TableCell();
                Button btUpdate = new Button();
                btUpdate.ID = "btUpdate";
                btUpdate.Text = "Update";
                btUpdate.Click += new EventHandler(btUpdate_Click);
                btUpdate.ToolTip = "Update";
                tcell2.Controls.Add(btUpdate);
                trow.Cells.Add(tcell2);
                tblDynamic.Rows.Add(trow);

                BuildImage(paramStack);
            }
            catch (Exception ex)
            {
                LabelError.Text = ex.ToString();
            }
        }
        #endregion

        /// <summary>
        /// Build image and hyperlink urls
        /// </summary>
        /// <param name="stack">ParameterStack to be passed to web services</param>
        protected void BuildImage(ParameterStack stack)
        {
            try
            {
                string parameterStr = sModelName;
                // add parameters with name and values
                foreach (Pic.Plugin.Parameter param in stack.ParameterList)
                {
                    parameterStr += "&" + param.Name + "=";
                    if (param.GetType().ToString().Equals("Pic.Plugin.ParameterDouble"))
                        parameterStr += ((ParameterDouble)param).Value.ToString();
                    else if (param.GetType().ToString().Equals("Pic.Plugin.ParameterBool"))
                        parameterStr += ((ParameterBool)param).Value.ToString();
                }
                // finally, set image url
                Image1.ImageUrl = "GetPicParamImage.aspx?name=" + parameterStr;
                // and hyperlink url
                btDownload.Click += new EventHandler(btDownload_Click);
            }
            catch (Exception ex)
            {
                LabelError.Text = ex.ToString();
            }
        }

        private void IterateControlsForUpdating(Control parent, Pic.Param.WebService.ParamService picParamServ)
        {
            foreach (Control child in parent.Controls)
            {
                if (child.GetType().ToString().Equals("System.Web.UI.WebControls.TextBox")
                    && child.ID.IndexOf("txt_") == 0)
                {
                }
                if (child.Controls.Count > 0)
                    IterateControlsForUpdating(child, picParamServ);
            }
        }

        private void IterateControlsForSaving(Control parent, Pic.Plugin.ParameterStack stack)
        {
            foreach (Control child in parent.Controls)
            {
                string sText = child.GetType().ToString();
                if (child.GetType().Equals( typeof(TextBox) ) && child.ID.IndexOf("txt_") == 0)
                {
                    TextBox tb = (TextBox)child;
                    foreach (Pic.Plugin.Parameter param in paramStack.ParameterList)
                    {
                        if (child.ID.Equals("txt_" + param.Name))
                            paramStack.SetDoubleParameter(param.Name, double.Parse(tb.Text));
                    }
                }
                else if (child.GetType().Equals(typeof(CheckBox)) && child.ID.IndexOf("cb_") == 0)
                {
                    CheckBox cb = (CheckBox)child;
                    foreach (Pic.Plugin.Parameter param in paramStack.ParameterList)
                    {
                        if (child.ID.Equals("cb_" + param.Name))
                            paramStack.SetBoolParameter(param.Name, cb.Checked);
                    }
                }
                if (child.Controls.Count > 0)
                    IterateControlsForSaving(child, stack);
            }
        }

        #region Event handlers
        protected void btUpdate_Click(object sender, EventArgs e)
        {
            try
            {
                Pic.Param.WebService.ParamService picParamServ = new Pic.Param.WebService.ParamService();
                paramStack = picParamServ.GetParameters(sModelName);
                IterateControlsForSaving(this, paramStack);
                BuildImage(paramStack);
            }
            catch (Exception ex)
            {
                LabelError.Text = ex.ToString();
            }
        }

        protected void btDownload_Click(object sender, EventArgs e)
        {
            try
            {
                Pic.Param.WebService.ParamService picParamServ = new Pic.Param.WebService.ParamService();
                IterateControlsForSaving(this, paramStack);

                string sFileFormat = cbDownload.Items[cbDownload.SelectedIndex].Text;
                System.Byte[] fileModel = picParamServ.GetExportFile(sFileFormat, sModelName, paramStack);
                // write file as response
                Response.ClearContent();
                Response.ClearHeaders();
                if ("dxf" == sFileFormat)
                    Response.AddHeader("content-disposition", string.Format("inline; filename={0}.dxf", sModelName));
                else if ("pdf" == sFileFormat)
                    Response.AddHeader("content-length", fileModel.GetLength(0).ToString());
                else
                    throw new Exception("Unknown file format.");
                Response.ContentType = "application/" + sFileFormat;
                Response.BinaryWrite(fileModel);
                Response.Flush();
                Response.Close(); 
            }
            catch (Exception ex)
            {
                LabelError.Text = ex.ToString();
            }
        }
        #endregion
    }
}
