﻿#region Using directives
using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;

using Pic.DAL;
using Pic.DAL.SqlServer;
#endregion

namespace Pic.Param.WebGallery
{
    public partial class GalleryWebForm : System.Web.UI.Page
    {
        #region Private data members
        private int _currentNodeId;
        #endregion

        #region Page initialization
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            if (!this.IsPostBack)
            {
                // load root nodes
                LoadRootNodes();
                // first node is selected
                DisplayDocumentTreeNode();
            }
        }

        protected void Page_Load(object sender, EventArgs e)
        {
        }
        #endregion

        #region Page layout management
        protected void DisplayDocumentTreeNode()
        {
            if (0 == _currentNodeId) return;
            try
            {
                PPDataContext db = new PPDataContext();
                // get docTreeNode from database
                Pic.DAL.SqlServer.TreeNode tn = Pic.DAL.SqlServer.TreeNode.GetById(db, _currentNodeId);
                // change ViewName text
                this.LblViewName.Text = tn.Name;
                this.LblViewDescription.Text = tn.Description;
                if (!tn.IsDocument)
                {
                    // show thumbnail view
                    this.MultiViewPane.ActiveViewIndex = 0;
                    // bind DataList with child DocTreeNode
                    dlImages.DataSource = tn.ChildrenDataSet(db);
                    dlImages.DataBind();
                    // disable ViewerControl
                    this.ViewerControl.DocumentId = 0;
                }
                else if (tn.IsDocument)
                {
                    // setup plugin viewer control
                    this.ViewerControl.DocumentId = (tn.Documents(db)[0]).ID;
                    // show document view
                    this.MultiViewPane.ActiveViewIndex = 1;
                }
            }
            catch (System.Exception ex)
            {
                LblException.Text = ex.ToString();
            }
        }
        #endregion

        #region TreeView related methods
        /// <summary>
        /// TreeView node click handler_currentNodeId
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void TreeViewDocument_SelectedNodeChanged(object sender, EventArgs e)
        {
            // get node id
            _currentNodeId = Convert.ToInt32(TreeViewDocuments.SelectedNode.Value);
            DisplayDocumentTreeNode();
        }

        protected void dlImages_SelectedItemChanged(object sender, EventArgs e)
        {
            try
            {
                // get the selected Id
                _currentNodeId =  Convert.ToInt32(dlImages.DataKeys[dlImages.SelectedIndex]);
                dlImages.SelectedIndex = -1;
                DisplayDocumentTreeNode();
            }
            catch (System.Exception ex)
            {
                LblException.Text = ex.ToString();
            }
        }

        /// <summary>
        /// Called to load TreeView root nodes when initializing page
        /// </summary>
        protected void LoadRootNodes()
        {
            try
            {
                PPDataContext db = new PPDataContext();
                List<Pic.DAL.SqlServer.TreeNode> rootNodes = Pic.DAL.SqlServer.TreeNode.GetRootNodes(db);
                foreach (Pic.DAL.SqlServer.TreeNode tn in rootNodes)
                {
                    System.Web.UI.WebControls.TreeNode tnRoot = new System.Web.UI.WebControls.TreeNode(tn.Name, tn.ID.ToString());
                    tnRoot.PopulateOnDemand = true;
                    tnRoot.SelectAction = TreeNodeSelectAction.Select;
                    tnRoot.ImageUrl = tn.IsDocument ? "~/TreeNodeImages/DOC.BMP" : "~/TreeNodeImages/CLSDFOLD.BMP";
                    tnRoot.ToolTip = tn.Description;
                    this.TreeViewDocuments.Nodes.Add(tnRoot);
                }

                // select first node
                if (TreeViewDocuments.Nodes.Count > 0)
                {
                    TreeViewDocuments.Nodes[0].Select();
                    _currentNodeId = Convert.ToInt32(TreeViewDocuments.Nodes[0].Value);
                }
            }
            catch (System.Exception ex)
            {
                LblException.Text = ex.ToString();
            }
        }
        /// <summary>
        /// Occurs when the server side tree needs loading some tree node childrens.
        /// </summary>
        /// <param name="sender">the TreeView</param>
        /// <param name="e">the tree view node arguments</param>
        protected void TreeViewDocument_TreeNodePopulate(object sender, TreeNodeEventArgs e)
        {
            try
            {
                // get docTreeNode from database
                PPDataContext db = new PPDataContext();
                Pic.DAL.SqlServer.TreeNode tn = Pic.DAL.SqlServer.TreeNode.GetById(db, Convert.ToInt32(e.Node.Value));
                // child nodes
                List<Pic.DAL.SqlServer.TreeNode> childNodes = tn.Childrens(db);
                foreach (Pic.DAL.SqlServer.TreeNode childNode in childNodes)
                {
                    System.Web.UI.WebControls.TreeNode tnChild = new System.Web.UI.WebControls.TreeNode(childNode.Name, childNode.ID.ToString());
                    tnChild.PopulateOnDemand = true;
                    tnChild.SelectAction = TreeNodeSelectAction.Select;
                    tnChild.ImageUrl = childNode.IsDocument ? "~/TreeNodeImages/DOC.BMP" : "~/TreeNodeImages/CLSDFOLD.BMP";
                    tnChild.ToolTip = childNode.Description;
                    e.Node.ChildNodes.Add(tnChild);
                }
            }
            catch (System.Exception ex)
            {
                LblException.Text = ex.ToString();
            }
        }
        #endregion

        #region Data list event handling
        /// <summary>
        /// Occurs when the server side DataList starts databinding. This event
        /// is used to intercept the databinding to create a new Image that
        /// uses the binary array value of the data within the DataList datasource
        /// </summary>
        /// <param name="sender">the datalist</param>
        /// <param name="e">the datalist DataListItemEventArgs</param>
        protected void dlImage_ItemDataBound(object sender, DataListItemEventArgs e)
        {
            try
            {
                // make sure this is an item in the data list (not header etc.)
                if ((e.Item.ItemType == ListItemType.Item) ||
                    (e.Item.ItemType == ListItemType.AlternatingItem))
                {
                    // get a reference to the image used for the bar in the row
                    System.Web.UI.WebControls.ImageButton img = (System.Web.UI.WebControls.ImageButton)(e.Item.FindControl("imgThumbnail"));
                    // get the nodeid 
                    string node_id = ((DataRowView)e.Item.DataItem).Row.ItemArray[0].ToString();
                    // set the source to the page that generates the thumbnail image
                    img.ImageUrl = ThumbFromID.PAGENAME + "?" + ThumbFromID.NODEID + "=" + node_id;
                }
            }
            catch (System.Exception ex)
            {
                LblException.Text = ex.ToString();
            }
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="source"></param>
        /// <param name="e"></param>
        protected void dlImages_ItemCommand(object source, DataListCommandEventArgs e)
        {
            try
            {
            }
            catch (System.Exception ex)
            {
                LblException.Text = ex.ToString();
            }
        }
        #endregion
    }
}
