﻿#region Using directives
using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using System.IO;

using Pic.Plugin;
using Pic.DAL.SqlServer;
#endregion

namespace Pic.Param.WebGallery
{
    public partial class PluginViewerControl : System.Web.UI.UserControl
    {
        #region Data members
        public int _documentId;
        protected DBauer.Web.UI.WebControls.DynamicControlsPlaceholder DCP;
        protected Dictionary<string, double> _majorations;
        private Pic.Plugin.Component _component;
        #endregion

        #region Plugin accessor
        protected Pic.Plugin.Component GetCurrentComponent()
        {
            if (null == _component)
            {
                // get data context
                PPDataContext db = new PPDataContext();
                // get the plugin
                Document doc = Document.GetById(db, _documentId);
                if (null == doc)
                    throw new System.Exception(string.Format("No valid document Id {0}", _documentId));
                using (ComponentLoader loader = new ComponentLoader())
                    _component = loader.LoadComponent(doc.File.Path);
                if (null == _component)
                    throw new System.Exception(string.Format("No valid plugin with document Id {0}", _documentId));
            }
            return _component; 
        }

        protected Dictionary<string, double> Majoration
        {
            get
            {
                if (null == _majorations)
                {
                    // get data context
                    PPDataContext db = new PPDataContext();
                    // get component
                    Pic.DAL.SqlServer.Component comp = Pic.DAL.SqlServer.Component.GetByDocumentID(db, _documentId);
                    // get cardboard profile
                    CardboardProfile profile = CardboardProfile.GetByName(db, DropDownListProfile.SelectedItem.Value);
                    // get list of majorations
                    _majorations = comp.GetDefaultMajorations(profile);
                }
                return _majorations;
            }
        }
        #endregion

        #region Public properties
        public int DocumentId
        {
            get { return _documentId; }
            set
            {
                _documentId = value;
                this.ViewState.Add("DocumentId", _documentId);
                _component = null;
                _majorations = null;
                if (0 != _documentId)
                    BuildParameterControls();
            }
        }
        #endregion

        #region System.Web.UI.UserControl overrides
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            try
            {
                PPDataContext db = new PPDataContext();
                // fill drop down profile list
                CardboardProfile[] cardboardProfiles = CardboardProfile.GetAll(db);
                DropDownListProfile.Items.Clear();
                foreach (CardboardProfile cbProfile in cardboardProfiles)
                {
                    DropDownListProfile.Items.Add(new ListItem("(" + cbProfile.Code + ")" + cbProfile.Name, cbProfile.Name));
                }
                
                // fill drop down download format list
                dlDownload.Items.Clear();
                string[] formatList = {"dxf", "pdf", "des"};
                foreach (string fileFormat in formatList)
                    dlDownload.Items.Add(fileFormat);

                btDownload.Click +=new EventHandler(btDownload_Click);

                this.DCP.ControlRestored += new DBauer.Web.UI.WebControls.DynamicControlEventHandler(this.DCP_ControlRestored);
                this.DCP.PreRestore += new System.EventHandler(this.DCP_PreRestore);
                this.DCP.PostRestore += new System.EventHandler(this.DCP_PostRestore);
            }
            catch (System.Exception ex)
            {
                LabelError.Text = ex.ToString();
            }
        }


        protected void Page_Load(object sender, EventArgs e)
        {
            // clear error message
            LabelError.Text = "";
            if (null != this.ViewState["DocumentId"])
                BuildParameterControls();
        }
        #endregion

        #region DCP handlers
        void DCP_ControlRestored(object sender, DBauer.Web.UI.WebControls.DynamicControlEventArgs e)
        {
        }
        void DCP_PreRestore(object sender, EventArgs e)
        {
        }
        void DCP_PostRestore(object sender, EventArgs e)
        { 
        }
        #endregion

        #region Parameter controls
        public void BuildParameterControls()
        { 
            try
            {
                _documentId = (int)this.ViewState["DocumentId"];
                if (0 == _documentId)  return;
                // get current plugin
                Pic.Plugin.Component component = GetCurrentComponent();
                // table for controls
                Table tblControls = new Table();
                tblControls.ID = "TableDynamic";
                DCP.Controls.Clear();
                DCP.Controls.Add(tblControls);
                tblControls.Rows.Clear();

                // show or hide label and combo for profiles
                bool hasMajo = false;
                foreach (Pic.Plugin.Parameter param in component.Parameters)
                {
                    for (int i = 1; i < 16; ++i)
                        if (param.Name == string.Format("m{0}", i))
                        {
                            hasMajo = true;
                            break;
                        }
                    if (hasMajo) break;
                }
                LabelProfile.Visible = hasMajo;
                DropDownListProfile.Visible = hasMajo;

                // create controls
                foreach (Pic.Plugin.Parameter param in component.Parameters)
                {
                    // check if parameter is a majoration
                    if (Majoration.ContainsKey(param.Name))
                        continue;
                    // add new row
                    TableRow tr = new TableRow();
                    tr.Width = 950;
                    tblControls.Rows.Add(tr);

                    // create controls
                    if (param.GetType() == typeof(ParameterDouble))
                    {
                        // create column 1
                        TableCell td1 = new TableCell();
                        td1.Width = 700;
                        // create label control dynamically
                        Label label1 = new Label();
                        label1.ID = "lbl_ " + param.Name;
                        label1.Text = param.Description;
                        label1.Width = 200;
                        // add control to the table cell
                        td1.Controls.Add(label1);

                        // create column2
                        TableCell td2 = new TableCell();
                        td2.Width = 250;
                        // create control dynamically
                        TextBox tb1 = new TextBox();
                        tb1.Width = 50;
                        tb1.ID = "txt_" + param.Name;
                        ParameterDouble paramDouble = (ParameterDouble)param;
                        tb1.Text = paramDouble.ValueDefault.ToString();
                        tb1.ToolTip = paramDouble.Name;
                        // add control to the table cell
                        td2.Controls.Add(tb1);
                        // add cells to the row
                        tr.Cells.Add(td1);
                        tr.Cells.Add(td2);
                    }
                    else if (param.GetType() == typeof(ParameterBool))
                    {
                        ParameterBool paramBool = param as ParameterBool;
                        TableCell td1 = new TableCell();
                        CheckBox cb1 = new CheckBox();
                        cb1.ID = "cb_" + param.Name;
                        cb1.Text = param.Description;
                        cb1.ToolTip = param.Name;
                        cb1.AutoPostBack = true;
                        cb1.Checked = paramBool.Value;
                        cb1.CheckedChanged += new EventHandler(UpdateImage);
                        // add control to the table cell
                        td1.Controls.Add(cb1);
                        // add cells to the row
                        tr.Cells.Add(td1);
                    }
                    else if (param.GetType() == typeof(ParameterInt))
                    {
                        // create column 1
                        TableCell td1 = new TableCell();
                        td1.Width = 700;
                        // create label control dynamically
                        Label label1 = new Label();
                        label1.ID = "lbl_ " + param.Name;
                        label1.Text = param.Description;
                        label1.Width = 200;
                        // create column2
                        TableCell td2 = new TableCell();
                        td2.Width = 250;
                        // create control dynamically
                        TextBox tb1 = new TextBox();
                        tb1.Width = 50;
                        tb1.ID = "txt_" + param.Name;
                        ParameterInt paramInt = param as ParameterInt;
                        tb1.Text = paramInt.ValueDefault.ToString();
                        tb1.ToolTip = paramInt.Name;
                    }
                    else if (param.GetType() == typeof(ParameterMulti))
                    { 
                    }
                    else
                        throw new System.Exception("Unknown parameter type");
                }
                // update button
                TableRow tr3 = new TableRow();
                tblControls.Rows.Add(tr3);
                TableCell td7 = new TableCell();
                tr3.Cells.Add(td7);
                TableCell td8 = new TableCell();
                tr3.Cells.Add(td8);
                Button btUpdate = new Button();
                td8.Controls.Add(btUpdate);
                btUpdate.ID = "btUpdate";
                btUpdate.Text = "Update";
                btUpdate.Click += new EventHandler(UpdateImage);
                btUpdate.ToolTip = "Update";

                BuildImageUrl(component.Parameters);
            }
            catch (System.Exception ex)
            {
                LabelError.Text = ex.ToString();
            }
        }

        private void IterateControlsForSaving(Control parent, Pic.Plugin.ParameterStack paramStack)
        {
            foreach (Control child in parent.Controls)
            {
                string sText = child.GetType().ToString();
                if (child.GetType().Equals(typeof(TextBox)) && child.ID.IndexOf("txt_") == 0)
                {
                    TextBox tb = (TextBox)child;
                    foreach (Pic.Plugin.Parameter param in paramStack)
                    {
                        if (child.ID.Equals("txt_" + param.Name))
                            paramStack.SetDoubleParameter(param.Name, double.Parse(tb.Text));
                    }
                }
                else if (child.GetType().Equals(typeof(CheckBox)) && child.ID.IndexOf("cb_") == 0)
                {
                    CheckBox cb = (CheckBox)child;
                    foreach (Pic.Plugin.Parameter param in paramStack)
                    {
                        if (child.ID.Equals("cb_" + param.Name))
                            paramStack.SetBoolParameter(param.Name, cb.Checked);
                    }
                }
                if (child.Controls.Count > 0)
                    IterateControlsForSaving(child, paramStack);
            }
        }
        #endregion

        #region Image generation
        protected void BuildImageUrl(ParameterStack stack)
        {
            // load majorations values in parameter stack
            foreach (Pic.Plugin.Parameter param in stack)
                if (Majoration.ContainsKey(param.Name))
                    stack.SetDoubleParameter(param.Name, Majoration[param.Name]);
            // initialize with document Id
            string parameterStr = "DocumentId=" + _documentId.ToString();
            // add each parameter
            Pic.Plugin.Component component = GetCurrentComponent();
            foreach (Pic.Plugin.Parameter param in stack.ParameterList)
            {
                parameterStr += "&" + param.Name + "=";
                if (param.GetType().ToString().Equals("Pic.Plugin.ParameterDouble"))
                    parameterStr += ((ParameterDouble)param).Value.ToString();
                else if (param.GetType().ToString().Equals("Pic.Plugin.ParameterBool"))
                    parameterStr += ((ParameterBool)param).Value.ToString();
            }
            // save reflexions
            parameterStr += "&" + "ReflexionX=" + GChkBReflexionX.Checked.ToString();
            parameterStr += "&" + "ReflexionY=" + GChkBReflexionY.Checked.ToString();
            parameterStr += "&" + "ShowQuotations=" + GChkBCotations.Checked.ToString();
 
            // set image url
            Image1.ImageUrl = "GetPicParamImage.aspx?" + parameterStr;

            // compute dimensions and lengths
            double width = 0.0, height = 0.0, lengthCut = 0.0, lengthFold = 0.0;
            Tools tools = new Tools(component, new ComponentSearchMethodDB());
            tools.Dimensions(stack, out width, out height, out lengthCut, out lengthFold);
            // dimensions
            LabelWidth.Text = string.Format("{0:0.00} m", width / 1000.0);
            LabelHeight.Text = string.Format("{0:0.00} m", height / 1000.0);
            // length
            LabelCut.Text = string.Format("{0:0.00} m", lengthCut / 1000.0);
            LabelFold.Text = string.Format("{0:0.00} m", lengthFold / 1000.0);
            LabelLengthTotal.Text = string.Format("{0:0.00} m", (lengthCut+lengthFold) / 1000.0);
        }
        #endregion

        #region Event handlers
        protected void UpdateImage(object sender, EventArgs e)
        {
            try
            {
                _majorations = null;
                Pic.Plugin.ParameterStack paramStack = GetCurrentComponent().Parameters;
                IterateControlsForSaving(this, paramStack);
                BuildImageUrl(paramStack);
            }
            catch (System.Exception ex)
            {
                LabelError.Text = ex.ToString();
            }
        }

        protected void btDownload_Click(object sender, EventArgs e)
        {
            try
            {
                PPDataContext db = new PPDataContext();
                Document doc = Document.GetById(db, _documentId);
                string sModelName = GetCurrentComponent().Name;
                // get parameters
                Pic.Plugin.ParameterStack paramStack = GetCurrentComponent().Parameters;
                IterateControlsForSaving(this, paramStack);
                // load majorations values in parameter stack
                foreach (Pic.Plugin.Parameter param in paramStack)
                    if (Majoration.ContainsKey(param.Name))
                        paramStack.SetDoubleParameter(param.Name, Majoration[param.Name]);
                // load file format
                string sFileFormat = dlDownload.Items[dlDownload.SelectedIndex].Text;
                // generate export file
                Pic.Plugin.Tools tools = new Tools(GetCurrentComponent(), new ComponentSearchMethodDB());
                tools.ReflexionX = GChkBReflexionX.Checked;
                tools.ReflexionY = GChkBReflexionY.Checked;
                tools.ShowCotations = GChkBCotations.Checked;
                byte[] bytes = tools.GenerateExportFile(sFileFormat, paramStack);
                // write file as response
                Response.ClearContent();
                Response.ClearHeaders();
                if ("dxf" == sFileFormat)
                    Response.AddHeader("content-disposition", string.Format("inline; filename={0}.dxf", sModelName));
                else if ("pdf" == sFileFormat)
                    Response.AddHeader("content-length", bytes.GetLength(0).ToString());
                else
                    throw new System.Exception("Unknown file format.");
                Response.ContentType = "application/" + sFileFormat;
                Response.BinaryWrite(bytes);
                Response.Flush();
                Response.Close();
            }
            catch (System.Exception ex)
            {
                LabelError.Text = ex.ToString();
            }
        }

        #endregion
    }
}