﻿#region Using directives
using System;
using System.Data;
using System.Web;
using System.Collections;
using System.Collections.Specialized;
using System.ComponentModel;

using System.Web.Services;
using System.Web.Services.Protocols;

using System.IO;
using System.Xml.Serialization;

using Pic.Plugin;
#endregion

namespace Pic.Param.WebService
{
    /// <summary>
    /// Summary description for ParamService
    /// </summary>
    [WebService(Namespace = "http://tempuri.org/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [ToolboxItem(false)]
    public class ParamService : System.Web.Services.WebService
    {
        #region Private fields
        private byte[] ImageBytes;
        #endregion

        #region Web service description methods
        /// <summary>
        /// Get web service version
        /// Actually, returns the assembly version
        /// </summary>
        /// <returns>a string version</returns>
        [WebMethod(
            Description = "Get the web service version (Version.Release.Patch.Build)")]
        public string GetVersion()
        {
            string sVersion = "1.0.0.0";
            return sVersion;
        }

        /// <summary>
        /// Get the list of plugin names
        /// </summary>
        /// <returns>a string collection</returns>
        [WebMethod(
            Description = "Get the list of plugin names.")]
        public StringCollection GetListOfAvailableModels()
        {
            // get plugin host           
            Pic.Plugin.PluginServices host = new Pic.Plugin.PluginServices();
            // get collection of plugin names
            return host.GetListOfPluginNames();
        }
        /// <summary>
        /// Get the list of possible output file formats
        /// </summary>
        /// <returns>a string collection</returns>
        [WebMethod(
            Description = "Get the list of file outputs.")]
        public StringCollection GetListOfOutputFileFormats()
        {
            StringCollection sc = new StringCollection();
            sc.Add("dxf");
            sc.Add("pdf");
            return sc;
        }
        #endregion

        #region Model related web service methods
        /// <summary>
        /// get the list of parameters for a given (by name) model
        /// </summary>
        /// <param name="sModelName">the model name which Parameter stack should be returned</param>
        /// <returns>a ParameterStack object</returns>
        [WebMethod(
            Description = "Get the list of parameters.")]
        [XmlInclude(typeof(ParameterDouble))]
        [XmlInclude(typeof(ParameterBool))]
        [XmlInclude(typeof(ParameterStack))]
        public Pic.Plugin.ParameterStack GetParameters(string sModelName)
        { 
            // get plugin            
            Pic.Plugin.PluginServices host = new Pic.Plugin.PluginServices();
            Pic.Plugin.IPlugin plugin = host.GetPluginByName(sModelName);

            return plugin.Parameters;
        }

        /// <summary>
        /// Create and returns an image of the specified model
        /// </summary>
        /// <param name="nWidth">Image width in pixels</param>
        /// <param name="nHeight">Image height in pixels</param>
        /// <param name="sModelName">Model name string</param>
        /// <returns>a byte array</returns>
        [WebMethod(
             Description = "Create an image of a specified model with a specified height and width, but with default parameters.")]
        public byte[] GetModelImageDefaultValues(int nWidth, int nHeight, string sModelName)
        {
            // test if model name is empty
            if (sModelName.Length == 0)
                throw new Exception("Model name must not be empty!");

            // get plugin
            Pic.Plugin.PluginServices host = new Pic.Plugin.PluginServices();
            Pic.Plugin.IPlugin plugin = host.GetPluginByName(sModelName);

            // build factory
            Pic.Factory2D.PicFactory factory = new Pic.Factory2D.PicFactory();

            if (plugin != null)
            {
                Pic.Plugin.ParameterStack stack = plugin.Parameters;
                plugin.CreateFactoryEntities(factory, stack);
            }

            // get bounding box
            Pic.Factory2D.PicVisitorBoundingBox visitor = new Pic.Factory2D.PicVisitorBoundingBox();
            factory.ProcessVisitor(visitor);

            // save
            Pic.Factory2D.PicGraphicsImage picImage = new Pic.Factory2D.PicGraphicsImage();
            picImage.ImageSize = new System.Drawing.Size(nWidth, nHeight);
            Pic.Factory2D.PicBox2D box = visitor.Box;
            box.AddMargin(5);
            picImage.DrawingBox = box;
            factory.Draw(picImage);

            ImageBytes = picImage.GetBuffer();

            return ImageBytes;
        }
        /// <summary>
        /// Create an image of a name specified model with a specified height
        /// </summary>
        /// <param name="nWidth">Image width in pixels</param>
        /// <param name="nHeight">Image height in pixels</param>
        /// <param name="sModelName">Model name string</param>
        /// <param name="stack">a valid parameter stack</param>
        /// <returns>a byte array</returns>
        [WebMethod(
            Description = "Create an image of a specified model with a specified height and width, and a set of parameters.")]
        public byte[] GetModelImage(int nWidth, int nHeight, string sModelName, Pic.Plugin.ParameterStack stack)
        {
            // get plugin
            Pic.Plugin.PluginServices host = new Pic.Plugin.PluginServices();
            Pic.Plugin.IPlugin plugin = host.GetPluginByName(sModelName);

            // build factory
            Pic.Factory2D.PicFactory factory = new Pic.Factory2D.PicFactory();

            if (plugin != null)
                plugin.CreateFactoryEntities(factory, stack);

            // get bounding box
            Pic.Factory2D.PicVisitorBoundingBox visitor = new Pic.Factory2D.PicVisitorBoundingBox();
            factory.ProcessVisitor(visitor);

            // save
            Pic.Factory2D.PicGraphicsImage picImage = new Pic.Factory2D.PicGraphicsImage();
            picImage.ImageSize = new System.Drawing.Size(nWidth, nHeight);
            Pic.Factory2D.PicBox2D box = visitor.Box;
            box.AddMargin(5);
            picImage.DrawingBox = box;
            factory.Draw(picImage);

            ImageBytes = picImage.GetBuffer();

            return ImageBytes;
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sFileFormat">File format string (e.g. "dxf", "pdf"...)</param>
        /// <param name="sModelName">Model name string</param>
        /// <param name="stack">a valid parameter stack</param>
        /// <returns>a byte array</returns>
        [WebMethod(
            Description = "Generate file from specified model with a specified set of parameters.")]
        public byte[] GetExportFile(string sFileFormat, string sModelName, Pic.Plugin.ParameterStack stack)
        {
            // get plugin
            Pic.Plugin.PluginServices host = new Pic.Plugin.PluginServices();
            Pic.Plugin.IPlugin plugin = host.GetPluginByName(sModelName);

            // build factory
            Pic.Factory2D.PicFactory factory = new Pic.Factory2D.PicFactory();

            if (plugin != null)
                plugin.CreateFactoryEntities(factory, stack);

            // get bounding box
            Pic.Factory2D.PicVisitorBoundingBox visitorBoundingBox = new Pic.Factory2D.PicVisitorBoundingBox();
            factory.ProcessVisitor(visitorBoundingBox);
            Pic.Factory2D.PicBox2D box = visitorBoundingBox.Box;
            // add margins : 5 % of the smallest value among height 
            box.AddMarginHorizontal(box.Width*0.05);
            box.AddMarginVertical(box.Height*0.05);

            // get dxf file content
            Pic.Factory2D.PicVisitorOutput visitor = null;
            if ("dxf" == sFileFormat)
                visitor = new Pic.Factory2D.PicVisitorDxfOutput();
            else if ("pdf" == sFileFormat)
                visitor = new Pic.Factory2D.PicVisitorPdfOutput(box);

            visitor.Title = sModelName;
            visitor.Author = "TREEDIM";

            factory.ProcessVisitor(visitor);
            return visitor.GetResultByteArray();
        }
        #endregion
    }
}
